#include "HIDDevice_Keyboard_objects.h"
#include "HIDDevice_Keyboard_resources.h"

//--------------------- User code ---------------------//
#include "stdint.h"

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////** Constants and Variables **////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// MMC module connections
sbit Mmc_Chip_Select           at GPIOD_ODR.B3;
// eof MMC module connections

sfr sbit PSW at GPIOB_ODR.B15;

static const _COLOR_PAPER = 0xF77E;
static const _COLOR_BLACK = 0;
static const _COLOR_WHITE = 0xFFFF;
static const _COLOR_FONT  = 0x632C;//0x9CD3;

static const _RAW_MAX_LENGTH = 17;
static const _FILE_NAME_MAX_LENGTH = 11;

static const _COLOR_CIRCLE_WAIT_0 = 0xDEFB;
static const _COLOR_CIRCLE_WAIT_1 = 0xCE79;
static const _COLOR_CIRCLE_WAIT_2 = 0x8C51;
static const _COLOR_CIRCLE_WAIT_3 = 0x528A;

const _FALSE = 0;
const _TRUE  = 1;

static const OK    = 0;
static const ERROR = 1;

static const uint8_t _MODE_EDIT = 0;
static const uint8_t _MODE_SAVE = 1;

static uint8_t modeMark;
static uint8_t curLine;

static uint8_t hidEvent;

static TLabel* textLine[9];

typedef struct {
  TLabel* nameLabel;
  TImage* img;
} TDialogBox;
static TDialogBox dialogBox;

void HidDevInit();
void HidDevReportReceived(uint8_t* rptData);

typedef struct{
  void(*DeviceInit)();
  void(*ReportReceived)(uint8_t* rptData);
} TUSBHostHIDHandler;

TUSBHostHIDHandler USBHost_HIDHandler = {
  HidDevInit,
  HidDevReportReceived
};

TUSBHostHIDHandler* USBHost_HIDHandlerPtr = &USBHost_HIDHandler;


////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////** Declarations of functions **///////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void WelcomeScreen();
void InitTimer();
void InitVar();
void InitMcu();
void TurnOffEffect();
void AddCharToStr(uint8_t* str, uint8_t chr);
void RemoveCharFromStr(uint8_t* str);
uint8_t AddCharToText(uint8_t chr);
uint8_t RemoveCharFromText();
uint8_t CheckPlugOutIn();
void StartWaitAnimation();
void EndWaitAnimation();
void UpdateWaitProgress();
void ClearLineLabels();
void SaveFile();
void SaveProcessData(uint8_t dat);
void HIDEventHandler(uint8_t event);

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////** Interrupts **////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


void USB0Interrupt() iv IVT_INT_OTG_FS {
  USBHost_IntHandler();
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
// Use for Drawing Spinner Animation

void Timer2_interrupt() iv IVT_INT_TIM2 {
  TIM2_SR.UIF = 0;
  UpdateWaitProgress();
}


////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////** Function definitions **///////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Draw Welcome screen

extern TScreen* CurrentScreen;
void WelcomeScreen() {
  Start_TP();
  Delay_ms(3000);
  TurnOffEffect();
  InitVar();
  InitTimer();
  // Demo of Wait Screen
  DrawScreen(&ScreenWait);
  StartWaitAnimation();
  Delay_ms(1000);
  InitMcu();
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
// Initialization of MCU modules, and variables

void InitVar() {

  curLine = 0;
  hidEvent = 255;

  // Initialization of array of labels, which represent Notepad Text
  textLine[0] = &LabelRow0;
  textLine[1] = &LabelRow1;
  textLine[2] = &LabelRow2;
  textLine[3] = &LabelRow3;
  textLine[4] = &LabelRow4;
  textLine[5] = &LabelRow5;
  textLine[6] = &LabelRow6;
  textLine[7] = &LabelRow7;
  textLine[8] = &LabelRow8;

  // Initialize field for Save Dialog Box
  dialogBox.nameLabel = &LabelDialogBox;
  dialogBox.img       = &ImageDialogBox;

}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
// Initialize Timer for drawing Progress Bar Wait animation

void InitTimer() {
  RCC_APB1ENR.TIM2EN = 1;
  TIM2_CR1.CEN = 0;
  TIM2_PSC = 119;
  TIM2_ARR = 60000;
  NVIC_IntEnable(IVT_INT_TIM2);
  TIM2_DIER.UIE = 1;
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
// Initialise rest of MCU modules

void InitMcu() {
  GPIO_Alternate_Function_Enable(&_GPIO_MODULE_OTG_FS_PA89_10_11_12);

  // Initialize SPI module
  SPI3_Init_Advanced(_SPI_FPCLK_DIV64, _SPI_MASTER | _SPI_8_BIT |
                     _SPI_CLK_IDLE_LOW | _SPI_FIRST_CLK_EDGE_TRANSITION |
                     _SPI_MSB_FIRST | _SPI_SS_DISABLE | _SPI_SSM_ENABLE | _SPI_SSI_1,
                     &_GPIO_MODULE_SPI3_PC10_11_12);

  // Initialization of USB module
  GPIO_Digital_Output(&GPIOB_BASE, _GPIO_PINMASK_15);
  PSW = 1;
  Delay_ms(100);
  PSW = 0;

  USBHost_HIDInit();
  USBHost_RegisterDeviceEventHandler(HIDEventHandler);
  USBHost_Init();
  NVIC_IntEnable(IVT_INT_OTG_FS);

  Sound_Init(&GPIOE_ODR, 14);
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
// Simple TurnOffDisplay Effect

void TurnOffEffect() {
char i;
  for(i = 0; i < 159; i++) {
    LineBlackUp.First_Point_Y++;
    LineBlackUp.Second_Point_Y++;
    LineBlackDown.First_Point_Y--;
    LineBlackDown.Second_Point_Y--;
    DrawLine(&LineBlackUp);
    DrawLine(&LineBlackDown);
  }
  Delay_ms(70);
  for(i = 0; i < 119; i++) {
    TFT_Dot(i, 160, 0);
    TFT_Dot(240-i, 160, 0);
    Delay_ms(1);
  }
  Delay_ms(400);
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
// Check for Device event

void HIDEventHandler(uint8_t event){
  if(event == _USB_HOST_EVENT_DEVICE_DISCONNECTED){
    hidEvent = _USB_HOST_EVENT_DEVICE_DISCONNECTED;
  }
  else if(event == _USB_HOST_EVENT_DEVICE_CONNECTED){
    hidEvent = _USB_HOST_EVENT_DEVICE_CONNECTED;
  }
  else if(event == _USB_HOST_EVENT_DEVICE_ENUMERATED){
    hidEvent = _USB_HOST_EVENT_DEVICE_ENUMERATED;
  }
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
// Handling spinner animation

static uint8_t waitCircle_index = 0;
TCircle* waitCircleArr[8] = {&CircleWait0, &CircleWait1, &CircleWait2, &CircleWait3,
                             &CircleWait4, &CircleWait5, &CircleWait6, &CircleWait7};
void UpdateWaitProgress() {
  waitCircleArr[waitCircle_index]->Color       = _COLOR_CIRCLE_WAIT_0;
  DrawCircle(waitCircleArr[waitCircle_index]);
  
  waitCircleArr[(waitCircle_index+1)%8]->Color = _COLOR_CIRCLE_WAIT_1;
  DrawCircle(waitCircleArr[(waitCircle_index+1)%8]);
  
  waitCircleArr[(waitCircle_index+2)%8]->Color = _COLOR_CIRCLE_WAIT_2;
  DrawCircle(waitCircleArr[(waitCircle_index+2)%8]);
  
  waitCircleArr[(waitCircle_index+3)%8]->Color = _COLOR_CIRCLE_WAIT_3;
  DrawCircle(waitCircleArr[(waitCircle_index+3)%8]);
  
  waitCircleArr[(waitCircle_index+4)%8]->Color = _COLOR_PAPER;
  DrawCircle(waitCircleArr[(waitCircle_index+4)%8]);
  
  waitCircleArr[(waitCircle_index+5)%8]->Color = _COLOR_PAPER;
  DrawCircle(waitCircleArr[(waitCircle_index+5)%8]);
  
  waitCircleArr[(waitCircle_index+6)%8]->Color = _COLOR_PAPER;
  DrawCircle(waitCircleArr[(waitCircle_index+6)%8]);
  
  waitCircleArr[(waitCircle_index+7)%8]->Color = _COLOR_PAPER;
  DrawCircle(waitCircleArr[(waitCircle_index+7)%8]);

  waitCircle_index++;
  if(waitCircle_index == 8)
    waitCircle_index = 0;
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
// Clear all Notepad Text

void ClearLineLabels() {
  uint8_t i;
  curLine = 0;
  for(i = 0; i < 9; i++)
  *(textLine[i]->Caption) = '\0';
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
// Adding char to string

void AddCharToStr(uint8_t* str, uint8_t chr) {
  str += strlen(str);
  *(str++) = chr;
  *str = '\0';
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
// Removing char from string

void RemoveCharFromStr(uint8_t* str) {
  str += strlen(str) - 1;
  *str = '\0';
}
  
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
// Adding char to Notepad Text

uint8_t AddCharToText(uint8_t chr) {

  // Check if current line (label) reached max length?
  if(strlen(textLine[curLine]->Caption) >= _RAW_MAX_LENGTH) {
    // If user reached end of paper
    if(curLine == 8) {
      Sound_Play(1000, 30);
      return ERROR;
    }
    // Go to next line
    else
      curLine++;
  }

  // Add char to currnet line (label)
  AddCharToStr(textLine[curLine]->Caption, chr);
  DrawLabel(textLine[curLine]);
  
  return OK;
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
// Removing char from Notepad Text

uint8_t RemoveCharFromText() {

  // Check if current line (label) reached zero length?
  if(strlen(textLine[curLine]->Caption) == 0) {
    // If user reached start of paper
    if(curLine == 0) {
      Sound_Play(1000, 30);
      return ERROR;
    }
    // Go to previous line
    else
      curLine--;
  }
  
  // Clear current line (label)
  textLine[curLine]->Font_Color = _COLOR_PAPER;
  DrawLabel(textLine[curLine]);
  textLine[curLine]->Font_Color = _COLOR_BLACK;

  // Remove char from currnet line (label)
  RemoveCharFromStr(textLine[curLine]->Caption);
  DrawLabel(textLine[curLine]);
  
  return OK;
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
// Handle Save Mode (for write Notepad Text to MMC card)

void SaveFile() {
  uint8_t i;

  // If current mode is SAVE MODE?
  if(modeMark == _MODE_SAVE) {
  
    // Draw Save dialog box
    dialogBox.img->Visible = 1;
    DrawImage(dialogBox.img);
    strcpy(dialogBox.nameLabel->Caption, "Name.txt");
    dialogBox.nameLabel->Visible = 1;
    DrawLabel(dialogBox.nameLabel);

    // Until we exit from "Save Dialog Box" (wait for Enter key)
    while(modeMark == _MODE_SAVE) {
      // Handle pressed key
      USBHost_Main();
      
      // Check is device plug-out and if it is, go ScreenWait,
      // wait for plug-in, and return to ScreenNotepad
      if(CheckPlugOutIn() == _TRUE) {
        modeMark = _MODE_EDIT;
        dialogBox.img->Visible = 0;
        dialogBox.nameLabel->Visible = 0;
        ClearLineLabels();
        DrawScreen(&ScreenNotepad);
        return;
      }
    }
    
    // Enter key detected, clear Save Dialog Box, and go to MMC write proces
    dialogBox.img->Visible = 0;
    dialogBox.nameLabel->Visible = 0;
    BoxPaper.Visible = 1;
    DrawBox(&BoxPaper);
    
    // Start spinner animation
    StartWaitAnimation();
    
    ButtonMmcState.Visible = 1;
    strcpy(ButtonMmcState_Caption, "Search MMC");
    DrawButton(&ButtonMmcState);
    Delay_ms(1500);

    Mmc_Init();
    Delay_ms(100);
    
    if (!Mmc_Init()) {
    
      // Clear old text
      ButtonMmcState.Font_Color = _COLOR_PAPER;
      DrawButton(&ButtonMmcState);
      
      // Update and redraw text
      ButtonMmcState.Font_Color = _COLOR_FONT;
      strcpy(ButtonMmcState_Caption, "Search FAT16");
      DrawButton(&ButtonMmcState);
      Delay_ms(1500);
      
      // Check file system
      if (!Mmc_Fat_Init()) {
      
        // Clear old text
        ButtonMmcState.Font_Color = _COLOR_PAPER;
        DrawButton(&ButtonMmcState);
        
        // Update and redraw text
        ButtonMmcState.Font_Color = _COLOR_FONT;
        strcpy(ButtonMmcState_Caption, "Create file");
        DrawButton(&ButtonMmcState);
        Delay_ms(1500);
        
        // Try to create/open file with name entered
        if(Mmc_Fat_Assign(dialogBox.nameLabel->Caption, 0xA0) == 1) {
          Mmc_Fat_Rewrite();
          // Write all 9 line of Notepad into MMC card
          for(i = 0; i < 9; i++) {
            Mmc_Fat_Write(textLine[i]->Caption, strlen(textLine[i]->Caption));
          }
          
          // Clear old text
          ButtonMmcState.Font_Color = _COLOR_PAPER;
          DrawButton(&ButtonMmcState);
          
          // Update and redraw text
          ButtonMmcState.Font_Color = _COLOR_FONT;
          strcpy(ButtonMmcState_Caption, "Success");
          DrawButton(&ButtonMmcState);
          Delay_ms(1500);
        }
        // Wrong file name detected. File name should be in DOS 8.3 (file_name.extension) format
        else {
          ButtonMmcState.Font_Color = _COLOR_PAPER;
          DrawButton(&ButtonMmcState);
          ButtonMmcState.Font_Color = _COLOR_FONT;
          strcpy(ButtonMmcState_Caption, "Error");
          DrawButton(&ButtonMmcState);
          Delay_ms(1500);
        }
        
      }
      // Not detected FAT16
      else {
        ButtonMmcState.Font_Color = _COLOR_PAPER;
        DrawButton(&ButtonMmcState);
        ButtonMmcState.Font_Color = _COLOR_FONT;
        strcpy(ButtonMmcState_Caption, "FAT16 not found");
        DrawButton(&ButtonMmcState);
        Delay_ms(1500);
      }
    }
    // Not detected MMC
    else {
      ButtonMmcState.Font_Color = _COLOR_PAPER;
      DrawButton(&ButtonMmcState);
      ButtonMmcState.Font_Color = _COLOR_FONT;
      strcpy(ButtonMmcState_Caption, "MMC not found");
      DrawButton(&ButtonMmcState);
      Delay_ms(1500);
    }
    
    // Clear Notepad Text
    ClearLineLabels();
    curLine = 0;
    
    ButtonMmcState.Visible = 0;
    EndWaitAnimation();
    DrawBox(&BoxPaper);
    BoxPaper.Visible = 0;
    // Indicate end of "Save Proces"
    Sound_Play(1000, 30);
    Delay_ms(50);
    Sound_Play(1000, 30);
  }
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
// Check last HID Event

uint8_t CheckPlugOutIn() {
uint8_t drawNotepadScr_mark = _FALSE;

  // If device disconnected, go to ScreenWait, start spinner and wait for PlugIn
  //if ((hidEvent == _USB_HOST_EVENT_DEVICE_DISCONNECTED)&&(CurrentScreen != &ScreenWait)) {
  if ((hidEvent == _USB_HOST_EVENT_DEVICE_DISCONNECTED)) {
    DrawScreen(&ScreenWait);
    StartWaitAnimation();
    
    while(hidEvent != _USB_HOST_EVENT_DEVICE_ENUMERATED) {
      USBHost_Main();
    }
    
    // Stop spinner
    EndWaitAnimation();
    // Set "Draw Notepad screen" mark
    drawNotepadScr_mark = _TRUE;
    // Indicate device enumerated event
    Sound_Play(1000, 30);
  }
  
  // If device enumerated, and current screen is not ScreenNotepad
  else if((hidEvent == _USB_HOST_EVENT_DEVICE_ENUMERATED)&&(CurrentScreen != &ScreenNotepad)) {
    // Stop spinner
    EndWaitAnimation();
    // Set "Draw Notepad screen" mark
    drawNotepadScr_mark = _TRUE;
  }
  
  return drawNotepadScr_mark;
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
// Start spinner animation

void StartWaitAnimation() {
  waitCircleArr[0]->Color = _COLOR_CIRCLE_WAIT_0;
  waitCircleArr[1]->Color = _COLOR_CIRCLE_WAIT_1;
  waitCircleArr[2]->Color = _COLOR_CIRCLE_WAIT_2;
  waitCircleArr[3]->Color = _COLOR_CIRCLE_WAIT_3;
  waitCircleArr[4]->Color = _COLOR_PAPER;
  waitCircleArr[5]->Color = _COLOR_PAPER;
  waitCircleArr[6]->Color = _COLOR_PAPER;
  waitCircleArr[7]->Color = _COLOR_PAPER;
  waitCircle_index = 0;
  TIM2_CR1.CEN = 1;
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
// End spinner animation

void EndWaitAnimation() {
  TIM2_CR1.CEN = 0;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////** Key Decode **////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

static  const  uint8_t  Keyboard_Key[] = {
  0, 0, 0, 0, 'a', 'b', 'c', 'd', 'e', 'f',
  'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p',
  'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z',
  '1', '2', '3', '4', '5', '6', '7', '8', '9', '0',
  '\n', 0, 255, 0, ' ', '-', '=', '[', ']', 0,
  '\\', ';', '\'', 0, ',', '.', '/'
};

static  const  uint8_t  Keyboard_ShiftKey[] = {
  0, 0, 0, 0, 'A', 'B', 'C', 'D', 'E', 'F',
  'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P',
  'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z',
  '!', '@', '#', '$', '%', '^', '&', '*', '(', ')',
  '\n', 0, 255, 0, ' ', '_', '+', '{', '}', 0,
  '|', ':', '"', 0, '<', '>', '?'
};

const uint8_t KBD_LEFT_CTRL       = 0x01;
const uint8_t KBD_LEFT_SHIFT      = 0x02;
const uint8_t KBD_RIGHT_CTRL      = 0x10;
const uint8_t KBD_RIGHT_SHIFT     = 0x20;
const uint8_t KBR_MAX_NBR_PRESSED = 6;

const uint8_t TRUE   = 0;
const uint8_t FALSE  = 1;

void Keyboard_ProcessData(uint8_t dat) {
  // If curent mode is _MODE_EDIT, then write to Notepad Text
  if(modeMark == _MODE_EDIT) {
    if((isgraph(dat) || isspace(dat)) && (dat != 10))
      AddCharToText(dat);
    else if(dat == 255)
      RemoveCharFromText();
  }
  // If curent mode is _SAVE_EDIT, then write Edit Box on SaveDialogBox panel
  else if(modeMark == _MODE_SAVE) {
    SaveProcessData(dat);
  }
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
// Update SaveDialogBox panel

void SaveProcessData(uint8_t dat) {

    // If "Enter" key pressed exit from _MODE_SAVE
    if(dat == '\n') {
      modeMark = _MODE_EDIT;
    }
    // If "Backspace" key pressed, and there is some chars in EditBox, clear last char
    else if((dat == 255)&&(strlen(dialogBox.nameLabel->Caption) > 0)) {
      dialogBox.nameLabel->Font_Color = _COLOR_WHITE;
      DrawLabel(dialogBox.nameLabel);
      RemoveCharFromStr(dialogBox.nameLabel->Caption);
      dialogBox.nameLabel->Font_Color = _COLOR_BLACK;
      DrawLabel(dialogBox.nameLabel);
    }
    // If the "dat" is a printable or "Space", and _FILE_NAME_MAX_LENGTH not reached yet
    // add char to EditBox
    else if( (strlen(dialogBox.nameLabel->Caption) <= _FILE_NAME_MAX_LENGTH) && (isgraph(dat) || isspace(dat)) ) {
      AddCharToStr(dialogBox.nameLabel->Caption, dat);
      DrawLabel(dialogBox.nameLabel);
    }
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
// Decode HID report

void KeyBoard_Decode(uint8_t *pbuf) {
  static  uint8_t   shift, ctrl, save_mark;
  static  uint8_t   keys[KBR_MAX_NBR_PRESSED];
  static  uint8_t   keys_new[KBR_MAX_NBR_PRESSED];
  static  uint8_t   keys_last[KBR_MAX_NBR_PRESSED];
  static  uint8_t   key_newest;
  static  uint8_t   nbr_keys;
  static  uint8_t   nbr_keys_new;
  static  uint8_t   nbr_keys_last;
  uint8_t   i, j;
  uint8_t   output;


  /* Check if Shift or Ctrl key is pressed */
  shift = FALSE;
  ctrl  = FALSE;
  if ((pbuf[0] & KBD_LEFT_SHIFT) || (pbuf[0] & KBD_RIGHT_SHIFT)) {
    shift = TRUE;
  }
  if ((pbuf[0] & KBD_LEFT_CTRL) || (pbuf[0] & KBD_RIGHT_CTRL)) {
    ctrl = TRUE;
  }

  /* Check for the value of pressed key */
  for (i = 2; i < 2 + KBR_MAX_NBR_PRESSED; i++) {
    if ((pbuf[i] == 0x01) || (pbuf[i] == 0x02) || (pbuf[i] == 0x03)) {
      return;
    }
  }

  nbr_keys     = 0;
  nbr_keys_new = 0;
  for (i = 2; i < 2 + KBR_MAX_NBR_PRESSED; i++) {
    if (pbuf[i] != 0) {
      keys[nbr_keys] = pbuf[i];
      nbr_keys++;
      for (j = 0; j < nbr_keys_last; j++) {
        if (pbuf[i] == keys_last[j]) {
          break;
        }
      }

      if (j == nbr_keys_last) {
        keys_new[nbr_keys_new] = pbuf[i];
        nbr_keys_new++;
      }
    }
  }

  if (nbr_keys_new == 1) {
    key_newest = keys_new[0];

    if(key_newest > 56)
      return;

    if(ctrl == TRUE) {
      // If Ctrl+s is pressed
      if(Keyboard_Key[key_newest] == 's')
        modeMark = _MODE_SAVE;
    }
    else {
      // If Shift is pressed
      if (shift == TRUE) {
        output =  Keyboard_ShiftKey[key_newest];
      }
      else {
        output =  Keyboard_Key[key_newest];
      }
      Keyboard_ProcessData(output);
    }
  }
  else {
    key_newest = 0x00;
  }

  nbr_keys_last  = nbr_keys;
  for (i = 0; i < KBR_MAX_NBR_PRESSED; i++) {
    keys_last[i] = keys[i];
  }
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
// Handler for HID Report Receive

void HidDevReportReceived(uint8_t* rptData){
  KeyBoard_Decode(rptData);
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

void HidDevInit(){
  if (USBHost_HIDGetDeviceType() == _USB_HOST_HID_DEVICE_MOUSE){
    ;
  }
  else if (USBHost_HIDGetDeviceType() == _USB_HOST_HID_DEVICE_KEYB){
    ;
  }
}

//----------------- End of User code ------------------//

// Event Handlers